@@ -4,7 +4,7 @@
 
 package Convert::ASN1;
 
-# $Id: _encode.pm,v 1.13 2002/01/22 11:24:28 gbarr Exp $
+# $Id: _encode.pm,v 1.16 2002/08/19 23:51:38 gbarr Exp $
 
 BEGIN {
   local $SIG{__DIE__};
@@ -53,8 +53,9 @@ sub _encode {
     &{$encode[$op->[cTYPE]]}(
       $optn,
       $op,
-      $stash,
-      defined($var) ? $stash->{$var} : undef,
+      (UNIVERSAL::isa($stash, 'HASH')
+	? ($stash, defined($var) ? $stash->{$var} : undef)
+	: ({}, $stash)),
       $_[4],
       $op->[cLOOP],
       $path,
@@ -347,7 +348,8 @@ sub _enc_choice {
 
   my $stash = defined($_[3]) ? $_[3] : $_[2];
   for my $op (@{$_[1]->[cCHILD]}) {
-    my $var = $op->[cVAR];
+    my $var = defined $op->[cVAR] ? $op->[cVAR] : $op->[cCHILD]->[0]->[cVAR];
+
     if (exists $stash->{$var}) {
       push @{$_[6]}, $var;
       _encode($_[0],[$op], $stash, $_[6], $_[4]);
